package dev.flyfish.framework.r2dbc.operations;

import group.flyfish.fluent.chain.common.ExecutableSql;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 * 我们为实际开发提供了响应式链，避免写sql
 *
 * @author wangyu
 */
public interface R2dbcFluentOperations {

    /**
     * 查询单个
     *
     * @param sql        可执行sql
     * @param resultType 结果类型
     * @param <T>        泛型
     * @return 结果
     */
    <T> Mono<T> findOne(ExecutableSql sql, Class<T> resultType);

    /**
     * 查询单个
     *
     * @param sql        可执行sql
     * @param resultType 结果类型
     * @param <T>        泛型
     * @return 结果
     */
    <T> Flux<T> find(ExecutableSql sql, Class<T> resultType);

    /**
     * 查询分页对象
     *
     * @param sql 可执行sql
     * @param <T> 泛型
     * @return 结果
     */
    <T> Mono<Page<T>> find(ExecutableSql sql, Pageable pageable, Class<T> resultType);
}
